# from os.path import isfile
# from os.path import dirname
from markdown_predictions.trainer import get_data, get_path

# version_file = '{}/version.txt'.format(dirname(__file__))

# if isfile(version_file):
#     with open(version_file) as version_file:
#         __version__ = version_file.read().strip()



from markdown_predictions.clean_data import PreProcessor
from markdown_predictions.parse_data import LoadSalesData

import pandas as pd
import os


def get_data(data_path: str) -> pd.DataFrame:
    """ Load in and clean the data """
    load_data = LoadSalesData.load_in_files(data_path)
    pre_processor = PreProcessor(df=load_data.sales_data)
    pre_processor.clean_up_data()
    
    return pre_processor.df

def get_path():
    print(os.listdir())


if __name__ == "__main__":
    print("HELLO 🥸🥝")
    print(get_data("raw_data"))


""" This script cleans the dataframe """
from types import prepare_class
from numpy.core.numeric import True_
import pandas as pd

from markdown_predictions.parse_data import LoadSalesData


PRICE_COLS = ["price_PRE", "price_POST"]
SYMBOLS_TO_REMOVE = ["€", "%"]


class PreProcessor:
    
    def __init__(self, df: pd.DataFrame):
        self.df = df
        self.numeric_cols = []
        self.object_cols = []
    
    def drop_rows_without_reference(self):
        """ If no product reference drop row - likely to be the total rows """
        self.df = self.df[self.df.reference_PRE.notnull()]
    
    def drop_row_with_missing_entries(self):
        """ Drop all rows with a '-' within them """
        self.df = self.df[~(self.df == '-').any(axis=1)]
    
    def replace_decimal_in_price_cols(self):
        """ Replace the comma with a decimal point in price columns """
        self.df[PRICE_COLS] = self.df[PRICE_COLS].replace({r',': '.'}, regex=True)

    def remove_symbols(self):
        """ Replace symbols in columns, otherwise cannot convert to float """
        for symbol in SYMBOLS_TO_REMOVE:
            self.df = self.df.replace({rf'([ +]?){symbol}': ''}, regex=True)
        
    def make_columns_numeric(self):
        """ Try to make columns numeric, else leave as original type """
        for col in self.df.columns:
            try:
                self.df[col] = self.df[col].replace({r',': ''}, regex=True).astype(float)
                self.numeric_cols.append(col)
            except ValueError as e:
                # print(e)
                self.object_cols.append(col)
    
    def clean_up_data(self):
        """ Clean up the dataframe """
        self.drop_rows_without_reference()
        self.drop_row_with_missing_entries()
        self.remove_symbols()
        self.replace_decimal_in_price_cols()
        self.make_columns_numeric()        
                

if __name__ == "__main__":
    # Load in the data locally into a single dataframe
    loaded_data = LoadSalesData.load_in_files("raw_data")
    # Initiate pre-processing class instance
    pre_processor = PreProcessor(df=loaded_data.sales_data)
    pre_processor.clean_up_data()



COLUMN_MAPPING = {
    "Libellé OP Com.":"delivery",
    "Vitrine collection":"window_display",
    "Cible":"target",
    "Sous-Cible":"sub_target",
    "Sexe":"gender",
    "Catégorie":"product_category",
    "Famille":"family",
    "Sous-Famille":"sub_family",
    "Macro catégorie": "macro_category",
    "Nom Ref":"reference_name",
    "RefCol":"reference",
    "Lib. RefCol":"description",
    "Coloris":"color",
    "Matière":"material",
    "Saisonnalité":"seasonality",
    "PVC France":"price",
    "Rang":"weekly_rank",
    "CA Net TTCSem.N":"turnover",
    "CA Net TTCSem.-1N":"turnover_w_sub1",
    "CA Net TTCSem.-2N":"turnover_w_sub2",
    "CA Net TTCSem.-3N":"turnover_w_sub3",
    "Qté venduesSem.N":"quantity_sold",
    "Qté venduesSem.-1N":"quantity_sold_sub1",
    "Qté venduesSem.-2N":"quantity_sold_sub2",
    "Qté venduesSem.-3N":"quantity_sold_sub3",
    "Taux de remiseSem.N":"discount_rate",
    "Taux de remiseSem.-1N":"discount_rate_sub1",
    "Stock PDVSem.N":"store_stock",
    "Stk transitN":"stock_transit",
    "Stock PDV + Transit":"total_store_stock",
    "CouvertureSem.N":"weekly_cover",
    "Tx EcoulSem.N":"sellthrough",
    "CA Net TTCCumulN":"cum_turnover",
    "Taux de remiseCumulN":"cum_discount_rate",
    "Qté venduesCumulN":"cum_quantity_sold",
    "Nb RCTen venteN":"num_sizes",
    "Nb PDVavec RC en venteN":"num_stores",
    "Semaine 1ère vteN":"first_week_sale",
    "DéfilementN":"rate_of_sale",
    "Tx EcoulCumulN":"cum_sellthrough",
    "Stk DispoEntrepotN":"warehouse_stock",
    "Stk DispoRALCdeGenN":"zero_stock",
    "Stk Dispopour RéassortN":"avail_warehouse_stock",
    "%":"markdown"
}

COLUMN_SELECTION = {"PRE": ["delivery",
         "window_display",
         "target",
         "gender",
         "product_category",
         "family",
         "sub_family",
         "macro_category",
         "reference_name",
         "reference",
         "description",
         "color",
         "material",
         "seasonality",
         "price",
         "weekly_rank",
         "turnover",
         "sub_target",
         "turnover_w_sub1",
         "turnover_w_sub2",
         "turnover_w_sub3",
         "quantity_sold",
         "quantity_sold_sub1",
         "quantity_sold_sub2",
         "quantity_sold_sub3",
         "discount_rate",
         "discount_rate_sub1",
         "store_stock",
         "stock_transit",
         "total_store_stock",
         "weekly_cover",
         "cum_turnover",
         "cum_discount_rate",
         "cum_quantity_sold",
         "num_sizes",
         "num_stores",
         "first_week_sale",
         "rate_of_sale",
         "cum_sellthrough",
         "warehouse_stock",
         "zero_stock",
         "avail_warehouse_stock",
         "markdown"],
 "POST": ["reference",
          "seasonality",
          "price",
          "weekly_rank",
          "turnover",
          "turnover_w_sub1",
          "turnover_w_sub2",
          "turnover_w_sub3",
          "quantity_sold",
          "quantity_sold_sub1",
          "quantity_sold_sub2",
          "quantity_sold_sub3",
          "discount_rate",
          "discount_rate_sub1",
          "store_stock",
          "stock_transit",
          "total_store_stock",
          "weekly_cover",
          "cum_turnover",
          "cum_discount_rate",
          "cum_quantity_sold",
          "num_sizes",
          "num_stores",
          "first_week_sale",
          "rate_of_sale",
          "cum_sellthrough",
          "warehouse_stock",
          "zero_stock",
          "avail_warehouse_stock",
          "markdown"],
    "reference_keys": ["reference", "season"]
}


""" This runs a set of unit tests on parse_data.py which extracts the sales data into dataframes """
from markdown_predictions.parse_data import LoadSalesData

def test_extract_season():
    """ This tests extracting the season from the filename """
    file_name = "raw_data/SS20 W20 PRE.csv"
    assert LoadSalesData.extract_season(file_name) == "SS20"


""" Reads in the sales data and loads into a pandas dataframe """
import collections
import glob
from json import encoder
import re
import os
import logging
import pandas as pd

from selection import COLUMN_SELECTION, COLUMN_MAPPING


class LoadSalesData:
    
    def __init__(self, sales_data: pd.DataFrame):
        self.sales_data = sales_data
    
    @staticmethod
    def extract_season(file_path: str, file_name: str):
        """ Regex the file name to extract the season """
        # Define the regex expression
        regex_expr = re.compile(rf"{file_path}\/(?P<season>\w+) W.+")
        m = regex_expr.search(file_name)
            
        if m:
            return m.group('season')
        else:
            logging.critical(f"Season cannot be extracted from {file_name}.")
            return None

    @staticmethod
    def read_in_files(file_path: str, pre_post_toggle: str, column_mapping: dict, selected_columns: dict):
        """ Read in files from local directory """
        all_sales_data = []
        
        for seasonal_file in glob.glob(f'{file_path}/*{pre_post_toggle}*'):
            logging.info(f"Reading file: {seasonal_file}")
            
            # Parse csv into dataframe
            seasonal_data = pd.read_csv(seasonal_file, index_col=None, encoding='utf-8')

            # Remove all escape characters
            seasonal_data.columns = [col.replace('\r','').replace('\n','').lower() for col in seasonal_data.columns]
            
            # Rename the columns
            seasonal_data.rename(columns=column_mapping, inplace=True)
            
            # If not all selected columns in the csv, report & exit.
            if not set(seasonal_data.columns).issuperset(set(selected_columns)):
                print(f"Not all selected columns present in csv file: {seasonal_file}.")
                print(f"Missing columns: {set(selected_columns).difference(set(seasonal_data.columns))}\nExiting.")
                exit(1)
            
            # Select the columns
            seasonal_data = seasonal_data[selected_columns]
            
            # Add season as a column to the dataframe
            season = LoadSalesData.extract_season(file_path=file_path, file_name=seasonal_file)
            if not season:
                # If season unable to be extracted skip
                continue
            seasonal_data['season'] = season
            
            # Add suffix
            seasonal_data = seasonal_data.add_suffix(suffix=f"_{pre_post_toggle}")
            all_sales_data.append(seasonal_data)

        return pd.concat(all_sales_data, axis=0, ignore_index=True)
    
    @staticmethod
    def make_dict_lowercase(input_dict: dict):
        lower_case_dict = {}
        for k, v in input_dict.items():
            lower_case_dict[k.lower()] = v  
        return lower_case_dict

    @classmethod
    def load_in_files(cls, file_path: str):
        """ Load in Files """
        mapping = LoadSalesData.make_dict_lowercase(COLUMN_MAPPING)
        
        pre_sales_data = LoadSalesData.read_in_files(file_path=file_path, pre_post_toggle="PRE", column_mapping=mapping, selected_columns=COLUMN_SELECTION["PRE"])
        post_sales_data = LoadSalesData.read_in_files(file_path=file_path, pre_post_toggle="POST", column_mapping=mapping, selected_columns=COLUMN_SELECTION["POST"])
        
        sales_data = pd.merge(pre_sales_data,
                              post_sales_data,
                              left_on=[key + "_PRE" for key in COLUMN_SELECTION["reference_keys"]] ,
                              right_on=[key + "_POST" for key in COLUMN_SELECTION["reference_keys"]],
                              how="inner")

        sales_data.drop(columns=["season_POST"], inplace=True)
        return cls(sales_data)


if __name__ == "__main__":
    loaded_data = LoadSalesData.load_in_files("raw_data")
